/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	CLEMENT					*/
/*	Its purpose is to handle SMTP protocole	*/
/*	on SMTP server premises and crunch data	*/
/*	and get clearance from the main Mentor	*/
/*	server to pass the E-mail to local	*/
/*	smtp server (sendmail, postfixe, etc)	*/
/*						*/
/************************************************/
#include	<syslog.h>
#include	<string.h>
#include	<errno.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	"modtra.h"
#include	"modclm.h"
#include	"modchk.h"
#include	"modsnd.h"
#include	"modtrp.h"
#include	"lvleml.h"
#include	"lvlspm.h"
#include	"lvluid.h"
#include	"lvlchl.h"
#include	"unilck.h"
#include	"unieml.h"
#include	"unidns.h"
#include	"uninet.h"
#include	"subrou.h"
#include	"dbgmem.h"

extern char **environ;

typedef struct {	/*adv dispatcher struct		*/
	char *checker;	/*checker name			*/
	pid_t pid;	/*sending channel		*/
	char *msg;	/*message to send		*/
	}DISTYP;

static char curpath[300];
/*

*/
/************************************************/
/*						*/
/*	Procedure to display the program usage	*/
/*						*/
/************************************************/
static void usage()

{
(void) fprintf(stderr,"%s Version %s\n",appname,rou_getvers());
(void) fprintf(stderr,"usage:  %s "
		      "[-b base] "
		      "[-d debug] "
		      "[-f ] "
		      "[-h ] "
		      "[-v ] "
		      "\n",appname);
(void) fprintf(stderr,"\t\t-b base      : application working directory\n");
(void) fprintf(stderr,"\t\t-d debug     : debug level\n");
(void) fprintf(stderr,"\t\t-f           : foreground\n");
(void) fprintf(stderr,"\t\t-h           : display usage\n");
(void) fprintf(stderr,"\t\t-v           : verbose debug\n");
(void) exit(-1);
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to load the config definition	*/
/*	and return a data structure		*/
/*						*/
/************************************************/
static CHLTYP *prepmng(CHLTYP *mng,BOXTYP *boxe,TRATYP *track)

{
CNFTYP *cnf;

if ((cnf=cnf_getconfig())!=(CNFTYP *)0) {
  int i;
  LSTTYP **look;

  if (mng==(CHLTYP *)0) {
    mng=(CHLTYP *)calloc(1,sizeof(CHLTYP));
    mng->boxe=boxe;
    }
  if ((look=dev_getlst(mng->look,pro_smtp,cnf->bindlst))!=(LSTTYP **)0) {
    mng->look=dev_freelst(mng->look);
    mng->look=look;
    }
  for (i=0;i<3;i++) {
    char **mout;
    char **minp;

    mout=(char **)0;
    minp=(char **)0;
    switch (i) {
      case 0	:
        mout=&(mng->chkidmod);
	minp=&(cnf->chkidmod);
	break;
      case 1	:
        mout=&(mng->rmponder);
	minp=&(cnf->rmponder);
	break;
      case 2	:
        mout=&(mng->spmdsrv);
	minp=&(cnf->spmdsrv);
	break;
      }
    *mout=rou_freestr(*mout); 
    if (*minp!=(char *)0)
      *mout=strdup(*minp);
    }
  (void) dev_mullisten(mng->look);
  mng->frelais=cnf->frelais;
  cnf->frelais=(char **)0;
  mng->relais=cnf->relais;
  cnf->relais=(IPTYP **)0;
  mng->auxs=cnf->auxs;
  cnf->auxs=(AUXTYP **)0;
  mng->lastload=cnf->lastload;
  track->dfs=eml_freeonedfs(track->dfs);
  track->dfs=eml_mkdf("default",(char *)0,cnf->spamlim,
		      cnf->qualoc,cnf->quahead,cnf->quaextra);
  track->skrlst=cnf_freeskrlst(track->skrlst);
  track->skrlst=cnf_getskrlst(mng->auxs);
  track->chkidsrv=rou_freestr(track->chkidsrv);
  track->chkidsrv=strdup(cnf->chkidsrv);
  track->dfltcode=strdup(cnf->dfltcode);
  track->chkidmod=rou_freestr(track->chkidmod);
  track->chkidmod=strdup(cnf->chkidmod);
  cnf=cnf_freeconfig(cnf);
  }
return mng;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to start process to listen	*/
/*	on ALL listed interface			*/
/*						*/
/************************************************/
static void dolistening(TRATYP *track,LSTTYP **look,int reload)

{
if (look!=(LSTTYP **)0) {
  int i;

  (void) dev_mullisten(look);
  for (i=0;look[i]!=(LSTTYP *)0;i++) {
    int j;

    if (look[i]->handle<0)
      continue;
    for (j=0;j<look[i]->iteration;j++) {
      if (look[i]->process[j]==(pid_t)0) {
        look[i]->process[j]=tra_tracking(track,look[i],j);
        continue;
        }
      if (kill(look[i]->process[j],SIGCHECK)<0)
        look[i]->process[j]=(pid_t)0;
      else 
	if (reload==true) {
         (void) kill(look[i]->process[j],SIGHUP);
	 }
      }
    }
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to check child process and	*/
/*	make sure they are up and running.	*/
/*						*/
/************************************************/
static void checkchild(CHLTYP *mng)

{
#define OPDR	"clement.c:checkchild,"

static time_t lastcheck=(time_t)0;

time_t curtime;

(void) rou_nozombie();
curtime=rou_systime();
mng->newchild=false;
if ((lastcheck+2)<=curtime) {
  ECHTYP t;

  for (t=chl_sender;t<chl_unknown;t++) {
    if (rou_checkprocess(mng->pid[t])==false) {
      mng->newchild=true;
      if (lastcheck!=(time_t)0)
        (void) alert(0,"%s detect '%s' process missing",OPDR,stmng[t]);
      switch (t) {
	case chl_sender		:
  	  mng->pid[(int)t]=snd_manager(mng);
	  break;
	case chl_checker	:
  	  mng->pid[(int)t]=chk_manager(mng);
	  break;
	case chl_transponder	:
	  if (mng->rmponder!=(char *)0)
  	    mng->pid[(int)t]=trp_manager(mng->rmponder);
	  else	/*no transponder defined	*/
  	    mng->pid[(int)t]=(pid_t)1;
	  break;
	case chl_clamav		:
  	  mng->pid[(int)t]=clm_manager(mng);
	  break;
	default			:
	  (void) alert(0,"%s CHLTYP '%d' not expected",OPDR,(int)t);
	  break;
        }
      }
    }
  lastcheck=curtime;
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to stop all managed process 	*/
/*	and free the data structure.		*/
/*						*/
/************************************************/
static CHLTYP *dropmng(CHLTYP *mng)

{
if (mng!=(CHLTYP *)0) {

  (void) dev_mulclose(mng->look);
  (void) rou_childkill(mng->pid,NSRV,5);
  mng=chl_freechlmng(mng);
  }
return mng;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to do messages screening	*/
/*	received by main clement process.	*/
/*						*/
/************************************************/
static void dispatchmsg(CHLTYP *mng,BOXTYP *boxe,char *caller,char *msg)

{
#define	OPDI	"clement.c:dispatchmsg,"

typedef	enum	{
	d_getmng,	/*give current child pid*/
	d_unknown	/*unknow request	*/
	}DENUM;

static VOCTYP vocdis[]={
        {d_getmng,GETMNG,(const void *)0},
        {d_unknown,(char *)0,(const void *)0}
        };

char *locmsg;
VOCTYP *voc;

locmsg=strdup(msg);
if ((voc=rou_getvoca(vocdis,msg))!=(VOCTYP *)0) {
  switch (voc->code) {
    case d_getmng	:
      (void) chl_clmmngpid(mng,boxe,caller);
      break;
    default		:
      (void) alert(0,"%s Unknown message <%s>",OPDI,msg);
      if (ipc_postreply(boxe,caller,(char *)eml_ascstat(CMDBAD,msg))<0) 
        (void) alert(0,"%s Unable to reply message (error=<%s>)",
		        OPDI,strerror(errno));
      break;
    }
  }
(void) free(locmsg);
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to open binding channel and	*/
/*	handle requested comming from SMTP	*/
/*	client.					*/
/*						*/
/************************************************/
static void  cycle()

{
#define	MXBOX	1000	/*Max 1000 messages queued*/

BOXTYP *boxe;
CHLTYP *mng;
TRATYP *track;

boxe=ipc_getbox(MXBOX);
mng=(CHLTYP *)0;
track=tra_maketrack(boxe);
(void) settitle("%s main daemon -d%d -b%s",appname,debug,base);
while (quickout==false) {
  if ((mng=prepmng(mng,boxe,track))==(CHLTYP *)0) {
    (void) alert(0,"%s configuration file missing altogether (aborting)",
		    appname);
    }
  else {
    TIMESPEC timeout;

    timeout.tv_nsec=0;
    timeout.tv_sec=1;	/*1 sec slice		*/
    while (ipc_waitpost(boxe,&timeout)==true) {
      char *caller;
      char *msg;
      u_long msec;

      if ((msg=ipc_getpost(boxe,&caller,&msec))!=(char *)0) {
        (void) dispatchmsg(mng,boxe,caller,msg);
        caller=rou_freestr(caller);
        msg=rou_freestr(msg);
        }
      }
    (void) spm_isspamdup(mng->spmdsrv);
    (void) checkchild(mng);
    (void) dolistening(track,mng->look,mng->newchild);
    (void) ipc_cleanbox(mng->boxe);
    }
  }
(void) spm_closespamd();
mng=dropmng(mng);
track=tra_freetrack(track);
boxe=ipc_dropbox(boxe);
}
/*
^L
*/
/************************************************/
/*						*/
/*	Program Entry.				*/
/*						*/
/************************************************/
int main(int argc,char *argv[])

{
char c;
STDDBG *dbglog;

memleak=DMTRACE;
base=strdup(base);
(void) strncpy(curpath,argv[0],sizeof(curpath));
(void) dbg_setcurpath(argv[0]);
(void) openlog(appname,LOG_PID,0);
while ((c=getopt(argc,argv,"b:d:fhv"))!=EOF) {
  switch(c) {
    case   'b'  :
      (void) free(base);
      base=rou_mkbase(optarg);
      break;
    case   'd'  :
      debug=atoi(optarg);
      break;
    case   'f'  :
      foreground=true;
      break;
    case   'v'  :
      verbose=true;
      break;
    case   'h'  :
    default	:
      (void) usage();
      break;
    }
  }
(void) dbg_setcurbase(base);
if ((debug==0)||(verbose==true)) {
  char curvers[100];
  char msg[200];

  (void) snprintf(curvers,sizeof(curvers),"%s",rou_getvers());
  (void) strcpy(msg,"");
  switch (dns_chksafeversion(curvers,sizeof(curvers))) {
    case -1	:
       (void) snprintf(msg,sizeof(msg),"\nYour current %s version is <%s> "
				       "while the last released one is <%s>.\n"
				        "You should update your %s program",
				        appname,rou_getvers(),curvers,appname);
      break;
    case  0	:
      break;
    case +1	:
       (void) snprintf(msg,sizeof(msg),"\nYour current %s-%s is a BETA "
				       "version.\n"
				       "Please report any discrepancy or bugs "
				       "you may find to <clement@safe.ca>.\n",
				        appname,rou_getvers());
      break;
    }
  if (strlen(msg)>0) {
    (void) fprintf(stderr,"%s\n",msg);
    }
  }
(void) rou_chgid(appname);
if ((strlen(base)>0)&&(chdir(base)<0)) {
  (void) fprintf(stderr,"Unable to reach the base directory <%s>\n",base);
  return -1;
  }
(void) divedivedive(foreground);
(void) preptitle(argc,argv,environ);
if (lck_locking(appname,LCK_LOCK,5)==true) {
  dbglog=eml_switchstd(appname);
  (void) setjournal(APJRL,d_log,appname);
  (void) journalling(true,APJRL,"%s-%s started",appname,rou_getvers());
  (void) setjournal(EVJRL,d_log,"events");
  (void) rou_setalrm();
  (void) cycle();
  dbglog=eml_closeswitchstd(dbglog);
  (void) journalling(true,APJRL,"%s-%s stopped",appname,rou_getvers());
  (void) closejournals();
  (void) lck_locking(appname,LCK_UNLOCK,1);
  }
(void) closelog();
(void) free(base);
(void) cleantitle();
(void) dbg_dumpmem(appname,"");
return 0;
}
