/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	LVLCHL:					*/
/*	Define procedure to manage child process*/
/*	number.					*/
/*						*/
/************************************************/
#include	<stdio.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/stat.h>
#include	"subrou.h"
#include	"lvlchl.h"

#define	MNGSNDR	"MNGSND"/*Sender manager		*/
#define	MNGCHKR	"MNGCHK"/*Chekcer manager		*/
#define	MNGTRPD	"MNGTRP"/*Transponder manager		*/
#define	MNGCLAM	"MNGCLM"/*clamav  manager		*/

const char *stmng[]={MNGSNDR,MNGCHKR,MNGTRPD,MNGCLAM,(char *)0};
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to get a specific manager PID	*/
/*	number from the application main	*/
/*	process.				*/
/*						*/
/************************************************/
pid_t chl_getmngpid(BOXTYP *boxe,const char *mngname)

{
pid_t mngpid;
char *msgref;
char rqst[300];

mngpid=(pid_t)0;
(void) snprintf(rqst,sizeof(rqst),GETMNG);
if ((msgref=ipc_postmsg(boxe,getppid(),rqst))!=(char *)0) {
  TIMESPEC timeout;

  timeout.tv_sec=2;
  timeout.tv_nsec=0;
  while ((mngpid==(pid_t)0)&&(ipc_waitpost(boxe,&timeout)==true)) {
    char *rsp;

    if ((rsp=ipc_getreply(boxe,msgref))!=(char *)0) {
      char **lines;

      if ((lines=eml_splitmsg(rsp))!=(char **)0) {
        int i;
  
        for (i=0;lines[i]!=(char *)0;i++) {
	  if (strstr(lines[i],mngname)==lines[i]) {
	    char *ptr;
  
	    ptr=lines[i]+strlen(mngname);
            mngpid=(pid_t)atoi(ptr);
	    break;
            }
	  }
        lines=rou_freestrlst(lines);
        }
      rsp=rou_freestr(rsp);
      }
    }
  msgref=rou_freestr(msgref);
  }
return mngpid;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to send a manager PID list to	*/
/*	requesting child process.		*/
/*						*/
/************************************************/
void chl_sndmngpid(BOXTYP *boxe,char *caller,char *mngpids)

{
(void) ipc_postreply(boxe,caller,mngpids);
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to send ALL manager PID to 	*/
/*	Requesting child process.		*/
/*						*/
/************************************************/
void chl_clmmngpid(CHLTYP *mng,BOXTYP *boxe,char *caller)

{
char *mnglst;
ECHTYP e;

mnglst=(char *)0;
for (e=chl_sender;e<chl_unknown;e++) {
  mnglst=rou_memstrcat(mnglst,"%s %d\n",stmng[(int)e],mng->pid[(int)e]);
  }
if (mnglst!=(char *)0) {
  (void) ipc_postreply(boxe,caller,mnglst);
  mnglst=rou_freestr(mnglst);
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to free all memory used by	*/
/*	a child definition.			*/
/*						*/
/************************************************/
CHLTYP *chl_freechlmng(CHLTYP *mng)

{
if (mng!=(CHLTYP *)0) {
  mng->look=dev_freelst(mng->look);
  mng->auxs=cnf_freeauxs(mng->auxs);
  mng->rmponder=rou_freestr(mng->rmponder);
  mng->relais=net_freeiplst(mng->relais);
  mng->frelais=rou_freestrlst(mng->frelais);
  mng->spmdsrv=rou_freestr(mng->spmdsrv);
  mng->chkidmod=rou_freestr(mng->chkidmod);
  (void) free(mng);
  mng=(CHLTYP *)0;
  }
return mng;
}
