/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	LVLSPM:					*/
/*	Spam detection routine level.		*/
/*						*/
/************************************************/
#include	<time.h>
#include	<syslog.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<pwd.h>
#include	<sys/stat.h>
#include	<string.h>
#include	"subrou.h"
#include	"unieml.h"
#include	"gestls.h"
#include	"lvlspm.h"

/*defining spam server variable			*/
#define	CLEMENT_SPMDSRV	"SPMDSRV"

/*default spam daemon port			*/
#define	SPMDPRT		783

/*spam proctocol Level				*/
#define	SPAMPROT	"SPAMC/1.3"

/*maximun waiting for spamd to respond		*/
#define	MXSPMWAIT    10	

/*define the current spmassassin running state	*/
static int spamstat=false;
static char *spmuser=(char *)0;
/*

*/
/************************************************/
/*						*/
/*	Procedure to get the file saie in byte	*/
/*						*/
/************************************************/
static long fsize(FILE *fichier)

{
long taille;
struct stat bufstat;

taille=0;
if (fstat(fileno(fichier),&bufstat)<0) {
  (void) alert(0,"lvlspm.c:fsize, Unable to get file stat (bug?)");
  }
else
  taille=bufstat.st_size;
return taille;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to open an spam detector	*/
/*	daemon channel				*/
/*						*/
/************************************************/
static TLSFILE *openspam()

{
TLSFILE *tlsfile;
char *spminf;

tlsfile=(TLSFILE *)0;
if ((spminf=getenv(CLEMENT_SPMDSRV))!=(char *)0) {
  int port;
  char hostname[100];

  if (sscanf(spminf,"%95[^\n\r,],%d",hostname,&port)!=2) 
    (void) alert(0,"%s Unable to scan ENV:%s=<%s>","lvlspm.s:openspam,",
		   CLEMENT_SPMDSRV,spminf);
  else {
    tlsfile=tls_opentcp(hostname,port,MXSPMWAIT);
    }
  }
return tlsfile;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to collect the answer from	*/
/*	spam checker daemon			*/
/*						*/
/************************************************/
char *getspmansw(TLSFILE *fspam)

{
#define	OPGT	"lvlspm.c:getspmansw,"

char *finding;
int phase;
int proceed;

finding=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  char *ligne;

  /*waiting 300 seconds	*/
  if ((ligne=tls_waitforaction(30*MXSPMWAIT,fspam))==(char *)0) {
    (void) alert(0,"%s timeout waiting for spamd",OPGT);
    break;
    }
  switch(phase) {
    case   0	:
      if (strstr(ligne,"EX_OK")==(char *)0) {
	(void) tracing(TRALOG,0,"%s unexpected spamd answer <%s>",
				 OPGT,ligne);
	phase=999;	/*get out quick			*/
	}
      break;
    case   1	:
      if (strlen(ligne)>0) {
	float result;
	float limite;
	char code[100];

        if (sscanf(ligne,"Spam: %19s ; %f / %f",code,&result,&limite)!=3) {
	  (void) tracing(TRALOG,0,"%s Unexpected spamd answer format <%s>",
				   OPGT,ligne);
	  }
	else {
	  (void) snprintf(ligne,sizeof(ligne),"%4.2f/%4.2f",result,limite);
          finding=strdup(ligne);
	  }
	}
			/*no break			*/
    default	:	/*safe guard			*/
      proceed=false;
      break;
    }
  ligne=rou_freestr(ligne);
  phase++;
  }
return finding;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to contact spam detector and	*/
/*	request about the file spamming level.	*/
/*						*/
/************************************************/
char *asktodaemon(FILE *fichier,TLSFILE *fspam)

{
#define	MXSPMTAIL	(500*1024)

char *finding;
long taille;
int phase;
int proceed;

finding=(char *)0;
taille=fsize(fichier);
phase=0;
proceed=true;
if (taille>MXSPMTAIL) { 
  register int dummycnt;

  dummycnt=asprintf(&finding,"%4.2f/%4.2f",0.0,0.0);
  proceed=false;
  }
while (proceed==true) {
  switch(phase) {
    case   0	:	/*sending header to spammassassin	*/
      (void) tls_printf(fspam,"CHECK %s\r\n",SPAMPROT);
      (void) tls_printf(fspam,"User: %s\r\n",spmuser);
      (void) tls_printf(fspam,"Content-length: %ld\r\n",taille);
      (void) tls_printf(fspam,"\r\n");
      break;
    case   1	:	/*sending file core to spamassassin	*/
      while (taille>0) {
	int max;
        int collected;
        char buffer[1025];

	max=sizeof(buffer)-1;
	if (max>taille)
	  max=taille;
        (void) memset(buffer,'\000',sizeof(buffer));
	if ((collected=fread(buffer,1,max,fichier))<0) {
	  (void) alert(0,"levspm.c:asktodaemon, Unable to fully collect "
			 "file, remaining=%d/%d",taille,fsize(fichier));
	  proceed=false;
	  break;
          }
	(void) tls_write(fspam,buffer,collected);
        taille-=collected;
        }
      break;
    case   2	:	/*extracting returned information	*/
      finding=getspmansw(fspam);
      break;
    default	:	/*safe guard				*/	
      proceed=false;
      break;
    }
  phase++;
  }
return finding;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to detecte if the spam daemon	*/
/*	(spamassassin) is up and running.	*/
/*						*/
/************************************************/
void spm_isspamdup(char *spmdsrv)

{
#define	OPUP	"lvlspm.c:spm_isspamdup,"

#define	NXTCHECK   600	/*checking daemon every	*/
			/*10 minutes		*/

static time_t lastcheck=(time_t)0;

time_t curtime;

curtime=rou_systime();
if ((lastcheck+NXTCHECK)<=curtime) {
  int newstat;
  TLSFILE *fspam;

  newstat=false;
  if (spmdsrv==(char *)0) 
    (void) unsetenv(CLEMENT_SPMDSRV);
  else
    (void) setenv(CLEMENT_SPMDSRV,spmdsrv,1);
  if ((fspam=openspam())!=(TLSFILE *)0) {
    char *ligne;

    (void) tls_printf(fspam,"PING %s\r\n",SPAMPROT);
    if ((ligne=tls_waitforaction(30,fspam))==(char *)0) {
      (void) alert(0,"%s timeout waiting for spamd",OPUP);
      }
    else {
      int code;
      char info[100];

      code=64;
      if (sscanf(ligne,"%s %d",info,&code)==2) {
	if (code==0)
          newstat=true;
	}
      if (newstat==false)
        (void) alert(0,"Spamassassin return <%s> to PING",ligne);
      ligne=rou_freestr(ligne);
      }
    (void) tls_close(fspam);
    }
  else {
    if (lastcheck==(time_t)0) {
      (void) alert(0,"Spamassassin not accessible, either '%s' is not "
		     "defined or Spmassassin daemon is not up and "
		     "running on this host",CLEMENT_SPMDSRV);
      }
    }
  if ((spamstat!=newstat)||(lastcheck==(time_t)0)) {
    char *strstat;

    if (newstat==false)
      strstat="down!";
    else {
      struct passwd *pw;

      spmuser=rou_freestr(spmuser); 
      if ((pw=getpwuid(geteuid()))!=(struct passwd *)0)
        spmuser=strdup(pw->pw_name);
      else
        spmuser=strdup(appname);
      strstat="up and running";
      }
    (void) journalling(true,APJRL,"Spam Detector Daemon found %s",strstat);
    }
  spamstat=newstat;
  lastcheck=curtime;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free data used by 		*/
/*	spam detector use.			*/
/*						*/
/************************************************/
void spm_closespamd()

{
spmuser=rou_freestr(spmuser);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the spam detector	*/
/*	finding about the file.			*/
/*						*/
/************************************************/
char *spm_checklevel(char *uname)

{
char *finding;

finding=(char *)0;
if (spamstat==true) {
  TLSFILE *fspam;

  if ((fspam=openspam())!=(TLSFILE *)0) {
    FILE *fichier;

    if ((fichier=fopen(uname,"r"))!=(FILE *)0) {
      finding=asktodaemon(fichier,fspam);
      (void) fclose(fichier);
      }
    (void) tls_close(fspam);
    }
  }
return finding;
}
