/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	MODSND:					*/
/*	Module level routines to handle		*/
/*	local E-mail delivery checking for Email*/
/*	stored in spool directory and		*/
/*	transmitting the E_mail via sendmail.	*/
/*						*/
/************************************************/
#include	<stdlib.h>
#include	<unistd.h>
#include	<malloc.h>
#include	<syslog.h>
#include	<string.h>
#include	<errno.h>
#include	<signal.h>
#include	<string.h>
#include	"subrou.h"
#include	"unidir.h"
#include	"unieml.h"
#include	"unicac.h"
#include	"unilck.h"
#include	"gesque.h"
#include	"gespbs.h"
#include	"lvleml.h"
#include	"lvlchl.h"
#include	"modsnd.h"

#define	TITIDLE	"%s, sending control (idle)"
typedef	struct	{	/*Dispatched exec queue	*/
	char *emailid;	/*email id to dispatch	*/
	char *sndr;	/*server to address Q	*/
	char *spamperti;/*Email pertinence	*/
	char *subject;	/*Email subject		*/
	char **emails;	/*From and recipients	*/
	}DEXTYP;
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by an 	*/
/*	dexlst					*/
/*						*/
/************************************************/
static DEXTYP **freedexlst(DEXTYP ** dexlst)


{
if (dexlst!=(DEXTYP **)0) {
  int i;

  for (i=0;dexlst[i]!=(DEXTYP *)0;i++) {
    dexlst[i]->emails=rou_freestrlst(dexlst[i]->emails);
    dexlst[i]->spamperti=rou_freestr(dexlst[i]->spamperti);
    dexlst[i]->subject=rou_freestr(dexlst[i]->subject);
    dexlst[i]->sndr=rou_freestr(dexlst[i]->sndr);
    dexlst[i]->emailid=rou_freestr(dexlst[i]->emailid);
    (void) free(dexlst[i]);
    }
  (void) free(dexlst);
  dexlst=(DEXTYP **)0;
  }
return dexlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to store an 'trouble' email	*/
/*	within the trouble queue, to be checked	*/
/*	later on.				*/
/*						*/
/************************************************/
static void settrouble(char *emailid,char *origin)

{
#define	OPST	"modsnd.c:settrouble,"

const char *frmqueue;
char trbldir[300];
char onameext[300];
char onameque[300];
char tnameext[300];
char tnameque[300];

frmqueue="fromqueue";
(void) rou_mkname(trbldir,sizeof(trbldir)-10,d_spool,TRBLDIR,frmqueue,"");
(void) rou_createdirs(trbldir);
(void) snprintf(trbldir,sizeof(trbldir),"%s%s/",TRBLDIR,frmqueue);
(void) rou_mkname(onameext,sizeof(onameext),d_spool,origin,emailid,EMAILEXT);
(void) rou_mkname(onameque,sizeof(onameque),d_spool,origin,emailid,QUEUEEXT);
(void) rou_mkname(tnameext,sizeof(tnameext),d_spool,trbldir,emailid,EMAILEXT);
(void) rou_mkname(tnameque,sizeof(tnameque),d_spool,trbldir,emailid,QUEUEEXT);
if (link(onameext,tnameext)<0)
  (void) alert(0,"%s unable to link file name <%s> to name <%s> (bug?)",
		  OPST,onameext,tnameext);
if (link(onameque,tnameque)<0)
  (void) alert(0,"%s unable to link file name <%s> to name <%s> (bug!?)",
		  OPST,onameque,tnameque);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to remove an Email from a	*/
/*	queue.					*/
/*						*/
/************************************************/
static void rmqueue(char *emailid,char *queue)

{
char qnameext[300];
char qnameque[300];

(void) rou_mkname(qnameext,sizeof(qnameext),d_spool,queue,emailid,EMAILEXT);
(void) rou_mkname(qnameque,sizeof(qnameque),d_spool,queue,emailid,QUEUEEXT);
(void) unlink(qnameext);
(void) unlink(qnameque);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add a queue execution to	*/
/*	a serveur queue.			*/
/*						*/
/************************************************/
static DEXTYP **addtodex(DEXTYP **dlst,char *emailid,char *sndr,
			 char *from,char *subject,char *spamperti,char *to)

{
int found;

found=false;
if (dlst!=(DEXTYP **)0) {
  int i;

  for (i=0;dlst[i]!=(DEXTYP *)0;i++) {
    if (strcasecmp(dlst[i]->sndr,sndr)==0) {
      dlst[i]->emails=(char **)rou_addveclst((void **)dlst[i]->emails,
					     (void *)strdup(to));
      found=true;
      break;
      }
    }
  }
if (found==false) {
  DEXTYP *dexloc;

  dexloc=(DEXTYP *)calloc(1,sizeof(DEXTYP));
  dexloc->emailid=strdup(emailid);
  if (subject!=(char *)0)
    dexloc->subject=strdup(subject);
  dexloc->sndr=strdup(sndr);
  if (spamperti!=(char *)0)
    dexloc->spamperti=strdup(spamperti);
  dexloc->emails=(char **)rou_addveclst((void **)0,(void *)strdup(from));
  dexloc->emails=(char **)rou_addveclst((void **)dexloc->emails,
					(void *)strdup(to));
  dlst=(DEXTYP **)rou_addveclst((void **)dlst,(void *)dexloc);
  }
return dlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to transmit the list of all	*/
/*	email received by user to Mentor (for 	*/
/*	remote learning process purpose).	*/
/*						*/
/************************************************/
static void dosent(BOXTYP *boxe,pid_t chkrpid,char **info)

{
if (info!=(char **)0) {
  int i;
  char *msg;
  char *msgref;

  msg=rou_memstrcat((char *)0,"%s\n",SENT);
  for (i=0;info[i]!=(char *)0;i++) {
    char *locmsg;
    char *ptr;

    locmsg=strdup(info[i]); 
    if ((ptr=strchr(locmsg,'-'))!=(char *)0) {
      if ((ptr=strchr(ptr,'.'))!=(char *)0) {
	do {
	  if (*ptr==' ')
	    break;
	  *ptr=' ';
	  ptr++;
	  }
 	while ((*ptr)!='\000');
	}
      }
    msg=rou_memstrcat(msg,"%s\n",locmsg);
    (void) free(locmsg);
    }
  if ((msgref=ipc_postmsg(boxe,chkrpid,msg))!=(char *)0) {
    msgref=rou_freestr(msgref);
    }
  (void) free(msg);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if a remote host	*/
/*	is ready to accept E-mail for a specific*/
/*	domain					*/
/*						*/
/************************************************/
static char *hostwilling(SKRTYP *skr,char *email,int relais)

{
char *host;
PLDTYP pld;
CHKTYP *chk;
int phase;
int proceed;

host=(char *)0;
(void) memset(&pld,'\000',sizeof(PLDTYP));
chk=(CHKTYP *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*lets open the channel	*/ 
      if ((chk=eml_openauxiliary(skr,""))==(CHKTYP *)0) {
	proceed=false;	/*unable to access sndr	*/
	}
      break;
    case  1	:	/*is sndr a good one	*/
      switch (chk->control) {
	case c_local   :
	case c_standard:
	  break;
	case c_mentor  :/*never ever occur	*/
	  phase=999;	
	  break;
	}
      break;
    case  2	:	/*is user a good one	*/
      switch (eml_checkuser(chk,"0000","",email,relais,&pld)) {
	case CMDOK	:
	case RELAYOK	:
	  host=skr->hostname;
	  break;
	default		:
	  break;
	}
      break;
    default	:	/*SAFE guard		*/
      chk=eml_closeauxiliary(chk);
      pld.info.smstat.str=rou_freestr(pld.info.smstat.str);
      proceed=false;
      break;
    }
  phase++;
  }
return host;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to find out what is the sender*/
/*	to be used to dispatch an E-mail.	*/
/*						*/
/************************************************/
static char **expandrcpts(SKRTYP **skrlst,char *emailid,char **rcpts)

{
char **explst;

explst=(char **)0;
if (rcpts!=(char **)0) {
  int i;

  for (i=0;rcpts[i]!=(char *)0;i++) {
    DOMTYP *skrs;
    char **aliases;
    char *domain;
    int phase;
    int proceed;

    skrs=(DOMTYP *)0;
    aliases=(char **)0;
    domain=eml_strdomain(rcpts[i]);
    phase=1;
    proceed=true;
    while (proceed==true) {
      switch (phase) {
	case 0		:	/*No alias for that user	*/
          explst=(char **)rou_addveclst((void **)explst,
					(void *)strdup(rcpts[i]));
	  proceed=false;
	  break;
	case 1		:	/*do we have a checker		*/
          if ((skrs=eml_findskr(skrlst,domain,false))==(DOMTYP *)0)
	    phase=-1;		/*no -> standard proceeding	*/
	  break;
	case 2		:	/*is checker a local one?	*/
	  if (strcasecmp(skrs->sndr1->hostname,KEEPLOC)!=0)
	    phase=-1;		/*no -> standard proceeding	*/
	  (void) free(skrs);
	  break;
	case 3		:	/*lets expand the user liste	*/
	  if ((aliases=que_getonealiases(rcpts[i]))==(char **)0)
	    phase=-1;		/*lets take the Email address	*/
	  break;
	case 4		:	/*lets take the aliases list	*/
          if (aliases!=(char **)0) {
	    int j;

            (void) journalling(false,EVJRL,"Email-ID %s",emailid);
	    for (j=0;aliases[j]!=(char *)0;j++) {
              explst=(char **)rou_addveclst((void **)explst,
					    (void *)aliases[j]);
              (void) journalling(false,EVJRL,"\t<%s> aliased to <%s>",
					      rcpts[i],aliases[j]);
	      }
            (void) free(aliases);
	    }
	  break;
        default		:	/*SAFE Guard			*/
	  proceed=false;
	  break;
        }
      phase++;
      }
    }
  }
/*cleaning double E-mail within list		*/
if (explst!=(char **)0) {
  int i;
  char **tmplst;

  tmplst=explst;
  explst=(char **)rou_addveclst((void **)0,(void *)tmplst[0]);
  for (i=1;tmplst[i]!=(char *)0;i++) {
    int j;
    int found;

    found=false;
    for (j=0;explst[j]!=(char *)0;j++) {
      if (strcasecmp(explst[j],tmplst[i])==0) {
	(void) free(tmplst[i]);
	found=true;
	break;
	}
      }
    if (found==false) {
      explst=(char **)rou_addveclst((void **)explst,(void *)tmplst[i]);
      }
    }
  (void) free(tmplst); 
  }
return explst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to find out what is the sender*/
/*	to be used to dispatch an E-mail.	*/
/*						*/
/************************************************/
static char *locsndr(SKRTYP **skrlst,char *email,int relais,char **chkname)

{
char *sndr;
DOMTYP *skrs;
PLDTYP *payload;
char *domain;
int phase;
int proceed;

sndr=(char *)0;
*chkname=(char *)0;
skrs=(DOMTYP *)0;
payload=(PLDTYP *)0;
domain=eml_strdomain(email);
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*check if already known	*/
      if ((payload=cac_getentry(domain,pldaux))!=(PLDTYP *)0) {
	if ((sndr=payload->info.host[0])!=(char *)0) {
	  *chkname=payload->info.host[1];
	  proceed=false;
	  }
	}
      break;
    case  1	:	/*lists find sender list	*/
      if ((skrs=eml_findskr(skrlst,domain,relais))==(DOMTYP *)0)
	proceed=false;
      break;
    case  2	:	/*lets check firts sender	*/
      if (skrs->sndr1==(SKRTYP *)0)
	phase++;
      break;
    case  3	:	/*firts sender willing to carry	*/
      if ((sndr=hostwilling(skrs->sndr1,email,relais))!=(char *)0) {
	if (skrs->chkr1!=(SKRTYP *)0) {
	  *chkname=skrs->chkr1->hostname;
	  }
	phase=999;
	}
      break;
    case  4	:	/*lets check secondary sender	*/
      if ((skrs->sndr2==(SKRTYP *)0)||(relais==false))
	phase++;
      break;
    case  5	:	/*secondary sndr ready to carry	*/
      sndr=hostwilling(skrs->sndr2,email,relais);
      break;
    default	:	/*SAFE guard			*/
      if (sndr!=(char *)0) {
	PLDTYP pld;

	pld.type=pldaux;
	pld.info.host[0]=sndr;
	pld.info.host[1]=*chkname;
        (void) cac_setentry(domain,&pld);
	}
      (void) free(skrs);
      proceed=false;
      break;
    }
  phase++;
  }
return sndr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to open the SMTP server	*/
/*	and keep open as long there is file	*/
/*	to transmit.				*/
/*						*/
/************************************************/
static void dosending(char *queuedir,SKRTYP *skr)

{
#define	OPCY	"modsnd.c:dosending,"
#define	SNDPACE	60	/*Local sender server	*/
#define	MXCLICK 8	/*8 Email max		*/

int click;
STDDBG *dbglog;
CHKTYP *sender;
char **lst;
int phase;
int proceed;
char srvque[300];

click=0;
sender=(CHKTYP *)0;
lst=(char **)0;
dbglog=eml_switchstd(skr->hostname);
(void) rou_mkname(srvque,sizeof(srvque),d_spool,queuedir,"","");
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    /* opening channel to remote sender		*/
    case 0	:
      if ((sender=eml_openauxiliary(skr,""))==(CHKTYP *)0) {
        (void) alert(0,"%s SMTP sender <%s> not responding!",
			OPCY,skr->hostname);
	(void) rou_passetime(180);	/*restart delay	*/
	proceed=false;
	}
      break;
    /*checking if list of Email to resend	*/
    case 1	:
      if ((lst=dir_getlst(srvque,QUEUEEXT))!=(char **)0) {
	int i;

        click=0;
	i=0;
        while (lst[i]!=(char *)0) {
          (void) settitle("%s, Sending <%s> to <%s>",
			       appname,lst[i],skr->hostname);
	  if (eml_sendingauxiliary(sender,queuedir,lst[i])==false) {
	    phase=999;
            break;	/*lets try again later on	*/
	    }
	  i++;
	  }
	(void) fflush(stdout);
	(void) fflush(stderr);
        lst=rou_freestrlst(lst);
	}
      break;
    /*waiting for action			*/
    case 2	:
      (void) settitle("%s, Sending to <%s> in idle mode",
		       appname,skr->hostname);
      proceed=rou_passetime(30);
      click++;
      if (click<MXCLICK)
	phase=-2;	/*lets check dir again	*/
      break;
    /*sending Idle information			*/
    case 3	:
      if (eml_smtpidle(sender,SNDPACE)==true) {
        if (eml_sendcmd(sender->contact,CMDOK,RSET)==CMDOK) {
	  phase=-3;	/*lets check dir again	*/
	  }
	}
      break;
    /*lets close the channel			*/
    default	:	/*SAFE guard		*/
      sender=eml_closeauxiliary(sender);
      proceed=false;
      break;
    }
  phase++;
  if (rou_checkprocess(getppid())==false)
    quickout=true;
  if ((quickout==true)||(sigreload==true))
    proceed=false;
  }
dbglog=eml_closeswitchstd(dbglog);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to link E-mail QUEUE within	*/
/*	the proper server directory		*/
/*						*/
/************************************************/
static void ventile(DEXTYP **dexlst)

{
#define	OPVE	"modsnd.c:ventile,"

if (dexlst!=(DEXTYP **)0) {
  int i;

  for (i=0;dexlst[i]!=(DEXTYP *)0;i++) {
    DEXTYP *dex;
    FILE *fichier;
    char *srv;
    char dirname[300];
    char filename[350];
    char dataname[350];

    dex=dexlst[i];
    srv=strdup(dex->sndr);
    (void) rou_mkname(filename,sizeof(filename),d_spool,SNDRDIR,srv,"");
    (void) rou_createdirs(filename);
    (void) snprintf(dirname,sizeof(dirname),"%s%s/",SNDRDIR,srv);
    (void) rou_mkname(filename,sizeof(filename),d_spool,dirname,
						dex->emailid,QUEUEEXT);
    if ((fichier=fopen(filename,"w"))!=(FILE *)0) {
      int j;

      if (dex->spamperti!=(char *)0)
        (void) fprintf(fichier,"%s %s\n",SPAM,dex->spamperti);
      (void) fprintf(fichier,"%s %s\n",SMAILF,dex->emails[0]);
      (void) fprintf(fichier,"%s %s\n",SUBJECT,dex->subject);
      for (j=1;dex->emails[j]!=(char *)0;j++) 
        (void) fprintf(fichier,"%s %s\n",SRCPTT,dex->emails[j]);
      (void) fprintf(fichier,SQUIT);
      (void) fclose(fichier);
      (void) rou_mkname(filename,sizeof(filename),d_spool,dirname,
						  dex->emailid,EMAILEXT);
      (void) rou_mkname(dataname,sizeof(dataname),d_spool,QUEUDIR,
						  dex->emailid,EMAILEXT);
      if (link(dataname,filename)<0) {
	(void) alert(0,"%s Unable to link file <%s> to <%s> (error=<%s>)",
			OPVE,filename,dataname,strerror(errno));
        }
      }
    else {
      (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
		      OPVE,filename,strerror(errno));
      }
    (void) free(srv);
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check ONE SMTP serveur 	*/
/*	queue and start a connection with 	*/
/*	the remote server if file are pending	*/
/*	in the queue.				*/
/*						*/
/************************************************/
static void lookat(SKRTYP *skr)

{
if ((skr->skrpid==(pid_t)0)||(kill(skr->skrpid,SIGCHECK)<0)) {
  char **lst;
  char queuedir[200];
  char srvque[300];

  skr->skrpid=(pid_t)0;
  skr->stat=s_down;
  (void) snprintf(queuedir,sizeof(queuedir),"%s%s/",SNDRDIR,skr->hostname);
  (void) rou_mkname(srvque,sizeof(srvque),d_spool,queuedir,"","");
  (void) rou_createdirs(srvque);
  if ((lst=dir_getlst(srvque,QUEUEEXT))!=(char **)0) {
    time_t curtime;
    pid_t child;

    lst=rou_freestrlst(lst);
    curtime=rou_systime();
    if ((skr->lasttry+60)<curtime) {
      skr->lasttry=curtime;	/*It's time to try again	*/
      switch((child=rou_fork(false))) {
        case  -1      :
          (void) alert(0,"modsend.c:lookat, Unable to start send "
			 "process for server <%s> (error=<%s>)",
			  skr->hostname,strerror(errno));
          break;
        case   0      :
	  (void) dosending(queuedir,skr);
          (void) exit(0);
          break;
        default       :
	  (void) sleep(5);	/*to avoid calling avalanche	*/
	  skr->skrpid=child;
	  skr->stat=s_active;
          break;
        }
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to reload sender configuration*/
/*						*/
/************************************************/
static int sndreload(BOXTYP *boxe,pid_t *chkrpid)

{
#define	OPRE	"modsnd.c:trareload,"

int done;

done=true;
if (rou_checkprocess(*chkrpid)==false)
  *chkrpid=(pid_t)0;
if ((sigreload==true)||(*chkrpid==(pid_t)0)) {
  int try;

  sigreload=false;
  done=false;
  for (try=0;try<3;try++) {
    *chkrpid=chl_getmngpid(boxe,stmng[(int)chl_checker]);
    if (*chkrpid!=(pid_t)0) {
      done=true;
      break;
      }
    (void) alert(0,"%s Unable to get <%s> pid (trouble?)",
		    OPRE,stmng[(int)chl_checker]);
    (void) sleep(1);
    }
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to dispatch Queued E-mail to	*/
/*	various SMTP server according RCPT	*/
/*	information.				*/
/*						*/
/************************************************/
static char **dispsend(SKRTYP **skrlst,char *emailid)

{
#define	OPSE	"modsnd.c:dispsend,"

char **info;
int done;
MLTYP *ml;

info=(char **)0;
done=false;
if ((ml=que_getemails(QUEUDIR,emailid))!=(MLTYP *)0) {
  DEXTYP **dexlst;
  char *fchkr;
  char **rcpts;
  int i;

  dexlst=(DEXTYP **)0;
  fchkr=(char *)0;
  done=true;
  (void) locsndr(skrlst,ml->from,false,&fchkr);
  rcpts=expandrcpts(skrlst,emailid,ml->rcpts);
  for (i=0;rcpts[i]!=(char *)0;i++) {
    char *sender;
    char *chkr;
    int relayed;

    relayed=true;	/*aliases -> always relayed	*/
    if ((sender=locsndr(skrlst,rcpts[i],relayed,&chkr))!=(char *)0) {
      char *strinfo;

      if (chkr==(char *)0)
        chkr=fchkr;
      if (chkr!=(char *)0) {
        strinfo=rou_memstrcat((char *)0,"%s %s %s",chkr,emailid,rcpts[i]);
        info=(char **)rou_addveclst((void **)info,(void *)strinfo);
	}
      dexlst=addtodex(dexlst,emailid,sender,ml->from,ml->subject,ml->spamperti,rcpts[i]);
      }
    else {
      done=false;
      (void) journalling(false,EVJRL,"Dispatching %s",emailid);
      (void) journalling(false,EVJRL,"\tRCPT <%s> not accepted",rcpts[i]);
      (void) alert(0,"%s Unable to find sender for user <%s>",OPSE,rcpts[i]);
      }
    }
  if (dexlst!=(DEXTYP **)0) {
    (void) ventile(dexlst);
    dexlst=freedexlst(dexlst);
    }
  rcpts=rou_freestrlst(rcpts);
  ml=que_dropemails(ml);
  }
(void) journalling(true,EVJRL,"");
if (done==false)
  (void) settrouble(emailid,QUEUDIR);
(void) rmqueue(emailid,QUEUDIR);
return info;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for an E-mail showing	*/
/*	up in the E-mail directory and take	*/
/*	that E-mail to be sent to sendmail.	*/
/*						*/
/************************************************/
static void sndcycle(CHLTYP *mng)

{
#define OPSN	"modsnd.c:sndcycle,"

SKRTYP **skrlst;
pid_t chkrpid;

skrlst=cnf_getskrlst(mng->auxs);
chkrpid=(pid_t)0;
sigreload=true;
while (quickout==false) {
  int found;
  char **lst;
  char queuedir[300];

  found=false;
  if (sndreload(mng->boxe,&chkrpid)==false) {
    (void) alert(0,"%s giving up about reload (big trouble?)", OPSN);
    break;
    }
  (void) rou_mkname(queuedir,sizeof(queuedir),d_spool,QUEUDIR,"","");
  if ((lst=dir_getlst(queuedir,QUEUEEXT))!=(char **)0) {
    char **info;
    int i;

    info=(char **)0;
    found=true;
    for (i=0;lst[i]!=(char *)0;i++) {
      char **newinfo;

      if ((newinfo=dispsend(skrlst,lst[i]))!=(char **)0) {
        int j;

	for (j=0;newinfo[j]!=(char *)0;j++) {
          info=(char **)rou_addveclst((void **)info,(void *)newinfo[j]);
          }
        (void) free(newinfo);
        }
      }
    (void) dosent(mng->boxe,chkrpid,info);
    info=rou_freestrlst(info);
    lst=rou_freestrlst(lst);
    }
  if (skrlst!=(SKRTYP **)0) {
    int i;

    for (i=0;skrlst[i]!=(SKRTYP *)0;i++) {
      if (skrlst[i]->pref==p_sender)
        (void) lookat(skrlst[i]);
      }
    }
  (void) settitle(TITIDLE,appname);
  (void) rou_nozombie();
  if (rou_checkprocess(getppid())==false) {
    quickout=true;
    break;
    }
  if (found==false) {
    (void) cac_expireentries();
    (void) pbs_cleanentry();
    (void) sleep(10);
    }
  }
(void) cac_cleanentries();
skrlst=cnf_freeskrlst(skrlst);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fork a process and start	*/
/*	wait for E-mail to be send to sendmail.	*/
/*						*/
/************************************************/
pid_t snd_manager(CHLTYP *mng)

{
pid_t child;

switch((child=rou_fork(false))) {
  case  -1      :
    (void) alert(0,"modsend.c:snd_manager, Unable to start loc process "
		   "(error=<%s>)",strerror(errno));
    break;
  case   0      :
    (void) journalling(true,APJRL,"%s (process='%d')",
		         	  "sending control",getpid());
    (void) settitle(TITIDLE,appname);
    memleak=DMTRACE;	/*memory leak tracking (xtrem debugging)*/
    (void) dbg_setnoleak(1);
    (void) sndcycle(mng);
    (void) journalling(true,APJRL,"%s now exiting (process='%d')",
		         	  "sending control",getpid());
    (void) dbg_dumpmem(appname,"sndmanager");
    (void) sleep(1);
    (void) exit(0);
    break;
  default       :
    break;
  }
return child;
}
