/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	MODTRP:					*/
/*	Module level routines to handle		*/
/*	local respond to Remote Mentor regarding*/
/*	Local user (in case there is a 		*/
/*	'keeplocal' definition within  the file	*/
/*	clement.conf.				*/
/*						*/
/************************************************/
#include	<unistd.h>
#include	<stdlib.h>
#include	<string.h>
#include	<syslog.h>
#include	<errno.h>
#include	<signal.h>
#include	"subrou.h"
#include	"unidev.h"
#include	"gesque.h"
#include	"gescnf.h"
#include	"lvleml.h"
#include	"modtrp.h"

#define	TRPD	"Transponder"

typedef	enum	{	/*SMTP vocabulary code	*/
	t_ehlo,		/*code extension	*/
	t_id,		/*sending the ID	*/
	t_resend,	/*send Quarantined Email*/
	t_remove,	/*delete   ''      Email*/
	t_addalias,	/*adding an aliase	*/
	t_remalias,	/*removing an aliases	*/
	t_addusr,	/*adding a user		*/
	t_remusr,	/*removing an user	*/
	t_deldom,	/*deleting a domain	*/
	t_tls,		/*Transport Socket Layer*/
	t_reset,	/*reset current stat	*/ 
	t_noop,		/*operation NOOP	*/
	t_quit,		/*abort connection	*/
	t_unknown	/*unknown code		*/
	}CODTYP;

static VOCTYP trvoca[]={
		{t_ehlo,"EHLO",(const void *)0},
		{t_tls,"STARTTLS",(const void *)0},
		{t_id,CKID,(const void *)0},
		{t_resend,"RESEND",(const void *)0},
		{t_remove,"REMOVE",(const void *)0},
		{t_addalias,"ADDALIAS",(const void *)0},
		{t_remalias,"REMALIAS",(const void *)0},
		{t_addusr,ADDUSR,(const void *)0},
		{t_remusr,REMUSR,(const void *)0},
		{t_deldom,DELDOM,(const void *)0},
		{t_reset,"RSET",(const void *)0},
		{t_noop,"NOOP",(const void *)0},
		{t_quit,"QUIT",(const void *)0},
		{t_unknown,(char *)0,(const void *)0}
		};
/*

*/
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle Remote Mentor	*/
/*	Request to know if an user is existing.	*/
/*						*/
/************************************************/
static void gotrequest(TLSFILE *tls,char *ipnum,int port)

{
#define	MXTIME	120	/*120 second time out	*/
#define MXLINE	300	/*line max size		*/

CTTYP *cta;
int ckid;
CNFTYP *cnf;
int proceed;

cta=eml_gotcontact(tls);
ckid=false;
if ((cnf=cnf_readconfig())==(CNFTYP *)0) {
  (void) alert(0,"modtrp.c:gotrequest, Unable to load configuration!");
  return;
  }
(void) eml_setsession(tls,cta);
cta->listenip=strdup(ipnum);
cta->listenport=port;
cta->attime=strdup(rou_getstrtime(rou_systime()));
*(strrchr(cta->attime,':'))='\000';	/*removing seconds display	*/
(void) journalling(true,APJRL,"%s got contact from <%s/[%s]>",
			       TRPD,cta->peername,cta->ipremote);
(void) eml_settitle(cta);
(void) eml_printfcrlf(tls,"%d %s TRSPD %s-%s; %s",
                          SIGNON,cta->locname,appname,
		          rou_getvers(),rou_ascsystime(rou_systime()));
proceed=true;
while (proceed==true) {
  VOCTYP *voc;
  int efface;
  MNGTYP code;
  char *buffer;

  efface=true; 
  code=m_add;
  rou_nozombie();
  if ((buffer=tls_waitforaction(MXTIME,cta->tls))==(char *)0) {
    cta->code=s_timeout;
    break;
    }
  (void) que_cleanstring(buffer);
  cta->code=s_timeout;
  voc=rou_getvoca(trvoca,buffer);
  if ((voc==(VOCTYP *)0)||(voc->code==t_unknown)) {
    (void) eml_printfcrlf(cta->tls,eml_ascstat(SYNERR,buffer));
    break;
    }
  switch (voc->code) {
    case t_noop		:
      (void) eml_printfcrlf(cta->tls,"%d OK",CMDOK);
      break;
    case t_reset	:
      (void) eml_printfcrlf(cta->tls,"%d Reset state",CMDOK);
      (void) eml_resetcontact(cta);
      ckid=false;
      break;
    case t_id		:
      if (strcmp(cnf->dfltcode,buffer)==0)
	ckid=true;
      break;
    case t_resend	:
      efface=false;	/*NO BREAK	*/
    case t_remove	:
      if (ckid==true) {
        char email[MXLINE];
        char emid[MXLINE];

        if (sscanf(buffer,"%s %s",email,emid)==2)  
          (void) que_mngquarantine(true,email,emid,efface);
        (void) eml_printfcrlf(cta->tls,"%d",CMDOK);
        }
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_remalias	:
      code=m_rem;	/*NO BREAK	*/
    case t_addalias	:
      if (ckid==true) {
	int status;
	int taille;
	char *domain;
	char *name;
	char *aliases;

	status=CMDABRT;
        taille=strlen(buffer);
	domain=calloc(taille+1,sizeof(char));
	name=calloc(taille+1,sizeof(char));
	aliases=calloc(taille+1,sizeof(char));
        if (sscanf(buffer,"%s %s %s",domain,name,aliases)==3) {
	  status=CMDOK;
          (void) que_mngaliases(code,domain,name,aliases);
	  }
        (void) eml_printfcrlf(cta->tls,"%s",eml_ascstat(status,buffer));
	(void) free(aliases);
	(void) free(name);
	(void) free(domain);
        }
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_deldom	:
      if (ckid==true) {
	(void) que_deldom(buffer);
        (void) eml_printfcrlf(cta->tls,"%d",CMDOK);
	}
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_remusr	:
      code=m_rem;	/*NO BREAK	*/
    case t_addusr	:
      if (ckid==true) {
	int status;
	PASTYP *pi;

	status=BADPAR;
	if ((pi=que_getusrinfo(buffer))!=(PASTYP *)0) {
          (void) que_mngemails(code,pi);
	  status=CMDOK;
          }
        (void) eml_printfcrlf(cta->tls,"%s",eml_ascstat(status,buffer));
        }
      else
        (void) eml_printfcrlf(cta->tls,"%d",BADID);
      break;
    case t_tls		:
      (void) eml_printfcrlf(cta->tls,"%d Ready to start TLS",SIGNON);
      (void) tls_gotls(cta->tls,true,cnf->SSLciphrs);
      break;
    case t_ehlo		:
      if (eml_checkhelo(cta,buffer,'-')==true) {
        (void) eml_printfcrlf(cta->tls,"%d STARTTLS",CMDOK);
	}
      break;
    case t_quit		:
      (void) eml_printfcrlf(cta->tls,eml_ascstat(QUITOK,cta->locname));
      proceed=false;
      break;
    }
  buffer=rou_freestr(buffer);
  }
(void) journalling(true,APJRL,"%s contact lost with  <%s/[%s]>",
			       TRPD,cta->peername,cta->ipremote);
cta=eml_itsover(cta);
cnf=cnf_freeconfig(cnf);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to detect remote Mentor	*/
/*	connection.				*/
/*						*/
/************************************************/
static void transponder(LSTTYP *channel)

{
#define	OPTA	"modtrp.c:transponder,"

TLSFILE *tls;

if ((tls=tls_accept(channel))!=(TLSFILE *)0) {
  switch (channel->proto) {
    case pro_smtp	:
      (void) gotrequest(tls,channel->ipnum,channel->port);
      break;
    case pro_chkid	:
    case pro_unknown	:
      (void) alert(0,"%s undefined protocol",OPTA);
      break;
    }
  (void) tls_close(tls);
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to handle one communication	*/
/*	channel with a remote client. return	*/
/*	the foked process pid.			*/
/*						*/
/************************************************/
pid_t trpaction(LSTTYP *channel,int num)

{
#define ACTCYC	"%s now %s [%s:%d] (%02d/%02d/%04d) (pid='%05d')"

pid_t child;

switch((child=rou_fork(false))) { 
  case  -1	:
    (void) alert(0,"%s:action Unable to start %s process "
                   "(error=<%s>)",appname,TRPD,strerror(errno));
    break;
  case   0	:	
      {
      int proceed;
      u_int cycle;
  
      memleak=DMTRACE;
      (void) dbg_setnoleak(1);
      proceed=true;
      cycle=0;
      (void) journalling(true,APJRL,ACTCYC,TRPD,"Starting",
				    channel->ipnum,
		         	    channel->port,num+1,
				    channel->iteration,cycle,getpid());
      while (proceed==true) {
        int gotone;;

	gotone=0;
        (void) settitle("%s, %s on [%s:%d] (%02d/%02d/%04d)",
		         appname,TRPD,channel->ipnum,
		         channel->port,num+1,channel->iteration,cycle);
        (void) transponder(channel);
        if ((quickout==true)||(sigreload==true))
	  break;
        }
      (void) journalling(true,APJRL,ACTCYC,TRPD,"exiting",
				    channel->ipnum,
		         	    channel->port,num+1,
				    channel->iteration,cycle,getpid());
      /*Call to check any memory leak if memleak set to 'true',	*/
      /*request done in debug mode only.			*/
      (void) dbg_dumpmem(appname,"trpaction");
      memleak=DMFALSE;
      (void) closejournals();
      (void) sleep(1);
      }
    (void) exit(0);
    break;
  default	:
    break;
  }
return child;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to start an SMTP connection	*/
/*	listening connection for remote Mentor	*/
/*	to check local user existance,		*/
/*						*/
/************************************************/
static void trpcycle(char *interface)

#define	OPTR	"modtrp.c:trpcycle,"

{
LSTTYP **look;

look=(LSTTYP **)0;
if ((look=dev_getlst(look,pro_smtp,interface))!=(LSTTYP **)0) {
  (void) journalling(true,APJRL,"Local %s <%s> ready",TRPD,interface);
  while (quickout==false) {
    int i;

    (void) dev_mullisten(look);
    (void) rou_nozombie();
    for (i=0;look[i]!=(LSTTYP *)0;i++) {
      int j;

      if (look[i]->handle<0)
        continue;
      for (j=0;j<look[i]->iteration;j++) {
        if (look[i]->process[j]==(pid_t)0) {
          look[i]->process[j]=trpaction(look[i],j);
          continue;
          }
        if (kill(look[i]->process[j],SIGCHECK)<0)
          look[i]->process[j]=(pid_t)0;
        }
      }
    (void) sleep(1);
    }
  (void) dev_mulclose(look);
  look=dev_freelst(look);
  }
(void) sleep(1);	/*to avoid avalanche	*/
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fork a process and start	*/
/*	wait for request to be check with a	*/
/*	local/remote sendmail.			*/
/*						*/
/************************************************/
pid_t trp_manager(char *interface)

{
pid_t child;

switch((child=rou_fork(false))) {
  case  -1      :
    (void) alert(0,"modtrp.c:trp_manager, Unable to start process "
		   "(error=<%s>)",strerror(errno));
    break;
  case   0      :
    (void) journalling(true,APJRL,"%s on <%s> (process='%d')",
		         	  TRPD,interface,getpid());
    (void) settitle("%s, %s manager",appname,TRPD);
    (void) trpcycle(interface);
    (void) exit(0);
    break;
  default       :
    break;
  }
return child;
}
