/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	UNICAC:					*/
/*	Take Care of all user cache definition	*/
/*						*/
/************************************************/
#include	<malloc.h>
#include	<syslog.h>
#include	<string.h>
#include	<search.h>
#include	"subrou.h"
#include	"unicac.h"

typedef	struct	{
	time_t stamp;	/*status time stamp	*/
	char *recipient;/*recipient Email	*/
	PLDTYP payload;	/*payload information	*/
	}USRCTY;

/*cache valide time, default is 30 minutes	*/
static int curvalide=1800;
/*cache stucture root pointer			*/
static void *cache=(void *)0;
static time_t walktime=(time_t)0;
/*

*/
/************************************************/
/*						*/
/*	Procedure to create memory to define	*/
/*	an user entry.				*/
/*						*/
/************************************************/
static USRCTY *mkusrcty(char *recipient,PLDTYP *payload)

{
USRCTY *usr;
int i;

usr=(USRCTY *)calloc(1,sizeof(USRCTY));
i=0;
usr->stamp=rou_systime()+curvalide;
usr->recipient=strdup(recipient);
usr->payload.type=payload->type;
switch (usr->payload.type) {
  case pldsmstat	:
    usr->payload.info.smstat.status=payload->info.smstat.status;
    if (payload->info.smstat.str!=(char *)0)
      usr->payload.info.smstat.str=strdup(payload->info.smstat.str);
    break;
  case pldaux	:
    for (i=0;i<2;i++)
      if (payload->info.host[i]!=(char *)0)
        usr->payload.info.host[i]=strdup(payload->info.host[i]);
    break;
  }
return usr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used to define	*/
/*	an user entry.				*/
/*						*/
/************************************************/
static USRCTY *freeusrcty(USRCTY *usr)

{
int i;

i=0;
if (usr!=(USRCTY *)0) {
  switch (usr->payload.type) {
    case pldsmstat	:
      usr->payload.info.smstat.str=rou_freestr(usr->payload.info.smstat.str);
      break;
    case pldaux	:
      for (i=0;i<2;i++)
        usr->payload.info.host[i]=rou_freestr(usr->payload.info.host[i]);
      break;
    }
  usr->recipient=rou_freestr(usr->recipient);
  (void) free(usr);
  usr=(USRCTY *)0;
  }
return usr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to compare 2 USRCTY entry	*/
/*	return an integer according compare	*/
/*	order.					*/
/*						*/
/************************************************/
static int compare(const void *pa,const void *pb)

{
return strcmp(((USRCTY *)pa)->recipient,((USRCTY *)pb)->recipient);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to delete old entries within	*/
/*	cache structure.			*/
/*						*/
/************************************************/
static void delact(const void *nodep,const VISIT which,const int depth)

{
USRCTY *key;

switch(which)
  {
  case preorder:
  case postorder:
    break;
  case leaf:
  case endorder:
    key= *(USRCTY **)nodep;
    (void)tdelete(key,&cache,compare);
    key=freeusrcty(key);
    break;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to set the cache valide time	*/
/*						*/
/************************************************/
void cac_setvalide(time_t valide)

{
if (valide>60)
  curvalide=valide;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to insert a cache information	*/
/*	within the search tree.			*/
/*						*/
/************************************************/
void cac_setentry(char *recipient,PLDTYP *payload)

{
USRCTY *key;
USRCTY **found;

key=mkusrcty(recipient,payload);
found=(USRCTY **)tsearch((void *)key,&cache,compare);
if ((found!=(USRCTY **)0)&&(*found!=key)) {
  key=freeusrcty(key);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if a user is already	*/
/*	within cache.				*/
/*						*/
/************************************************/
PLDTYP *cac_getentry(char *recipient,PLDENUM type)

{
PLDTYP *payload;
USRCTY *key;
PLDTYP pld;
USRCTY **found;

payload=(PLDTYP *)0;
pld.type=type;
switch(type) {
  case pldsmstat	:
    pld.info.smstat.status=0;
    pld.info.smstat.str=(char *)0;
    break;
  case pldaux		:
    pld.info.host[0]=(char *)0;
    pld.info.host[1]=(char *)0;
    break;
  }
key=mkusrcty(recipient,&pld);
if ((found=(USRCTY **)tfind(key,&cache,compare))!=(USRCTY **)0) {
  payload=&((*found)->payload);
  }
key=freeusrcty(key);
return payload;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check for record need to be*/
/*	remove from the cache			*/
/*	within cache.				*/
/*						*/
/************************************************/
void cac_expireentries()

{
static time_t lastexp=(time_t)0;

walktime=rou_systime();
if ((lastexp+60)<=walktime) {
  (void) twalk(cache,delact);
  lastexp=walktime;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free the whole memory used	*/
/*	by the cache structure.			*/
/*						*/
/************************************************/
void cac_cleanentries()

{
(void) tdestroy(cache,(__free_fn_t)freeusrcty);
cache=(void *)0;
}
