/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	UNIIPC:					*/
/*	Take Care of all access to shared memory*/
/*						*/
/************************************************/
#include	<errno.h>
#include	<memory.h>
#include	<syslog.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include	<sys/shm.h>
#include	<sys/time.h>
#include	"subrou.h"
#include	"uniipc.h"

#ifdef	_SEM_SEMUN_UNDEFINED
union semun {
	int val;		/*<= value for SETVAL			*/
	struct semid_ds *buf;	/*<= buffer for IPC_STAT & IPC_SET	*/
	unsigned short *array;	/*<= array for GETALL & SETALL		*/
	struct seminfo *__buf;	/*<= buffer for IPC_INFO		*/
	};
#undef _SEM_SEMUN_UNDEFINED
#endif

typedef	enum   {	/*lock flag		*/
  	s_lock,		/*locking memory access	*/
	s_release	/*release memory access	*/
	}LCKFLG;

typedef	struct {/*defining shared memory info	*/
	int semid;	/*semaphore ID		*/
	short sm;	/*semaphore array	*/
	int shmid;	/*shared memory ID	*/
	void *shmem;	/*shared memory area	*/
	}SHRTYP;

static	SHRTYP **shrlst;/*shared memory infos	*/	
/*
^L
*/
/************************************************/
/*						*/
/*	Routine to duplicate semtimedop which	*/
/*	is not available in Linux kernel 2.4	*/
/*						*/
/************************************************/
static int mysemtimedop(int semid,struct sembuf *sops,
			unsigned nsops,TIMESPEC *timeout)

{
int statut;
int locerrno;
struct itimerval timer;
struct itimerval oldtimer;

timer.it_interval.tv_sec=0;
timer.it_interval.tv_usec=0;
timer.it_value.tv_sec=timeout->tv_sec;
timer.it_value.tv_usec=timeout->tv_nsec/1000;
(void) setitimer(ITIMER_REAL,&timer,(struct itimerval *)0);
/*such setting the semaphore won't wait for ever*/
statut=semop(semid,sops,nsops);
locerrno=errno;
timer.it_value.tv_sec=0;
timer.it_value.tv_usec=0;
(void) setitimer(ITIMER_REAL,&timer,&oldtimer);
timeout->tv_sec=oldtimer.it_value.tv_sec;
timeout->tv_nsec=oldtimer.it_value.tv_usec*1000;
errno=locerrno;
return statut;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine to localise shared information	*/
/*	within list.				*/
/*						*/
/************************************************/
static int getsharepos(void *ptr)

{
int position;

position=-1;
if (shrlst!=(SHRTYP **)0) {
  int i;

  for (i=0;shrlst[i]!=(SHRTYP *)0;i++) {
    if (shrlst[i]->shmem==ptr) {
      position=i;
      break;
      }
    }
  }
return position;
}
/*
^L
*/
/************************************************/
/*                                              */
/*	Routine to sync access to a specific	*/
/*	shared memory.				*/
/*                                              */
/************************************************/
int syncsharedaccess(void *ptr,LCKFLG action,struct timespec *timeout)

{
#define	OPSY	"uniipc.c:syncsharedaccess,"

int lockdone;
int position;
struct sembuf directives;

lockdone=false;
if ((position=getsharepos(ptr))<0) {
  (void) alert(0,"%s Unable to find shared memory for ptr '%08lx'!",OPSY,ptr);
  return false;
  }
directives.sem_num=0;
directives.sem_flg=SEM_UNDO;
switch (action) {
  case s_lock     :
    directives.sem_op=-1;
    while (lockdone==false) {
      int toolate;

      toolate=false;
      lockdone=true;
      if (mysemtimedop(shrlst[position]->semid,&directives,1,timeout)<0) {
        lockdone=false;
        switch (errno) {
          case EINTR	:
	    lockdone=quickout;	/*in case we receive a quick-out*/
            break;		/*else it is just an ALRM signal*/
          case EAGAIN	:	/*we reached a timeout!		*/
	    toolate=true;
            break;
          default		:
            (void) alert(0,"%s Unable to lock semaphore (errno=<%s>)",
			    OPSY,strerror(errno));
            break;
          }
	}
      if (toolate==true)
       break;
      }
    break;
  case s_release  :
    directives.sem_op=1;
    if (semop(shrlst[position]->semid,&directives,1)<0) {
      (void) alert(0,"%s Unable to Unlock semaphore (errno=<%s>)",
		      OPSY,strerror(errno));
      }
    lockdone=true;
    break;
  }
return (lockdone==true);
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine to assign ONE semaphore		*/
/*						*/
/************************************************/
static SHRTYP *getsem()

{
SHRTYP *shared;

int semid;

shared=(SHRTYP *)0;
if ((semid=semget((long)0,1,0600|IPC_CREAT))>=0) {
  struct semid_ds semid_ds;
  union semun arg;

  shared=(SHRTYP *)calloc(1,sizeof(SHRTYP));
  shared->semid=semid;  
  shared->sm=1;
  (void) semctl(shared->semid,1,SETALL,&(shared->sm));
  semid_ds.sem_perm.gid=rou_getappid(appname,true);
  semid_ds.sem_perm.uid=rou_getappid(appname,false);
  semid_ds.sem_perm.mode=0600;
  arg.buf=&semid_ds; 
  if (semctl(shared->semid,0,IPC_SET,arg)<0) {
    (void) alert(0,"uniipc.c:getsem, Unable to change ownershipore (error=<%s>",
		   strerror(errno));
    }
  }
else {
  (void) alert(0,"uniipc.c:getsem, Unable to get semaphore (error=<%s>",
		  strerror(errno));
  }
return shared;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine to release a semaphore		*/
/*						*/
/************************************************/
static SHRTYP *dropsem(SHRTYP *shared)

{
if (shared!=(SHRTYP *)0) {
  (void) semctl(shared->semid,1,IPC_RMID,0);
  (void) free(shared);
  shared=(SHRTYP *)0;
  }
return shared;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine to request shared memory to 	*/
/*	which access is controled by a semaphore*/
/*						*/
/************************************************/
void *ipc_getshrmem(size_t nmemb,size_t size)

{
SHRTYP *sharloc;
void *shrmem;

shrmem=(void *)0;
if ((sharloc=getsem())!=(SHRTYP *)0) {
  int shmid;

  if ((shmid=shmget(IPC_PRIVATE,size*nmemb,IPC_CREAT|0600))>=0) {
    sharloc->shmid=shmid;
    if ((sharloc->shmem=shmat(shmid,(void *)0,0))==(void *)-1) {
      (void) alert(0,"uniipc.c:getshrmem, Unable to assigne shared memory "
		     "(error=<%s>)",strerror(errno));
      (void) shmctl(shmid,IPC_RMID,0);
      sharloc=dropsem(sharloc);
      }
    else {
      shrmem=sharloc->shmem;
      shrlst=(SHRTYP **)rou_addveclst((void **)shrlst,(void *)sharloc);
      }
    }
  else {
    (void) alert(0,"uniipc.c:getshrmem, Unable to get shared memory "
		     "(error=<%s>)",strerror(errno));
    sharloc=dropsem(sharloc);
    }
  }
return shrmem;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine to release shared memory	*/
/*						*/
/************************************************/
void *ipc_dropshrmem(void *ptr)

{
if (ptr!=(void *)0) {
  int position;

  if ((position=getsharepos(ptr))>=0) {
    int i;
    SHRTYP *shareloc;

    shareloc=shrlst[position];
    (void) shmdt(ptr);
    (void) shmctl(shareloc->shmid,IPC_RMID,0);
    shareloc=dropsem(shareloc);
    for (i=position;shrlst[i]!=(SHRTYP *)0;i++)
      shrlst[i]=shrlst[i+1];
    }
  if (shrlst[0]==(SHRTYP *)0) {
    (void) free(shrlst);
    shrlst=(SHRTYP **)0;
    }
  ptr=(void *)0;
  }
return ptr;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine which is getting back when	*/
/*	access to shared memory is allowed.	*/
/*						*/
/************************************************/
int ipc_getshraccess(void *ptr,struct timespec *timeout)

{
return syncsharedaccess(ptr,s_lock,timeout);
}
/*
^L
*/
/************************************************/
/*						*/
/*	Routine which is giving back free access*/
/*	to shared memory.			*/
/*						*/
/************************************************/
int ipc_dropshraccess(void *ptr)

{
return syncsharedaccess(ptr,s_release,(struct timespec *)0);
}
